#ident	"$Header: /var/cvsroot/htdb/src/include/htdb.h,v 1.56 2004/05/14 20:09:04 del Exp $"

/*
	Copyright  1994-2004 David Whittemore
	<del@adjective.com>, all rights reserved.

	Permission to use, copy, modify, and non-commercially distribute
	this software and its documentation for any purpose is hereby granted
	without fee, provided that the above copyright notice appear in all
	copies and that both that copyright notice and this permission notice
	appear in supporting documentation.  No representations are made about
	the suitability of this software for any purpose.  It is provided
	"as is" without express or implied warranty.
 */
#ifndef	H_HTDB
#define	H_HTDB

/*
	autoconf/automake-build config file
 */
#include 	"htdbconfig.h"

#if ((defined(USE_MYSQL) && defined(HAVE_LIBMYSQLCLIENT)) || defined(USE_DBI))
	#define	USE_DATABASE
#elif	(defined(USE_DBI) && defined(HAVE_LIBDBI))
	#define	USE_DATABASE
#else
	#undef	USE_DATABASE
#endif

/*
	common application constants
 */
#define	LEN_GOKEY			10		/* how many characters are in database gokeys? */
#define	FCGI_KEEPALIVE		1000	/* how many pages does fastcgi serve before exiting? */
#define	BUF_LIMIT_TINY		1024	/* resource defs are at least this big */
#define	BUF_LIMIT_SMALL		4096	/* small general-purpose storage size */
#define	BUF_LIMIT_MEDIUM	8192	/* big general-purpose storage size */
#define	BUF_LIMIT_BIG		16384	/* resource buffers are incremented by this much */
#define	BUF_LIMIT_MAX		1638400	/* longest expanded-string allowable in varg */
#define	ARE_YOU_INSANE		"server running as root; function disabled"

#define	HASHTABSIZE_SMALL	71		/* hash.c constants - the 20th prime */
#define	HASHTABSIZE_MEDIUM	409     /* hash.c constants - the 80th prime */
#define	HASHTABSIZE_BIG		3571    /* hash.c constants - the 500th prime */
#define	HASHTABSIZE_MAX		3571

#define BASE64Uint32    unsigned long

#include	<stdio.h>
#include	<stdlib.h>
#include	<ctype.h>
#include	<fcntl.h>
#include	<limits.h>

#ifdef HAVE_MALLOC_H
	#include	<malloc.h>
#endif

#include	<math.h>
#include	<values.h>
#include	<stdarg.h>
#include	<string.h>
#include	<strings.h>
#include	<syslog.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	<sys/time.h>
#include	<syslog.h>
#include	<time.h>
#include	<unistd.h>
#include	<dirent.h>
#include	<dlfcn.h>

#ifdef HAVE_CRYPT_H
	#include	<crypt.h>
#endif // HAVE_CRYPT_H

#include	<signal.h>

#if	(defined(HAVE_LIBFCGI) && defined(USE_FASTCGI))
	#include	"fcgi_stdio.h"
#endif

typedef	long	int_t;
typedef	long long	long_t;
typedef enum { BOOL_FALSE = 0, BOOL_TRUE = 1 }	bool_t;

typedef struct tdata *Pdata;
typedef struct tdata {					/* ternary bucket type */
	char	*key;
	void	*value;
	int		size,
			limit;
}	Tdata;

typedef struct tnode *Pnode;
typedef struct tnode {					/* ternary type */
	char splitchar;
	Pnode lokid, eqkid, hikid;
} Tnode;

typedef struct bucket_t {				/* hash bucket type */
	char	*key;
	void	*value;
	int		size,
			limit,
#ifdef	USE_NAMESPACE
			namespace_id,
#endif
			chainlen;
	struct bucket_t *next;
}	*Bucket;

typedef struct hashtable_t {			/* hashtable type */
	int		idx,
			valuesFreeable,
			size,
			numelements,
			maxchainlen;
	Bucket	bkt,
			ht_table[HASHTABSIZE_MAX];
	void	(*destructor)(void *);
}	*Hashtable;

struct llitem_t {						/* linked-list type */
	void	*value;
	struct llitem_t	*next;
};
typedef struct llitem_t *LLItem;

struct linkedlist_t {
	LLItem	     first;
	LLItem	     item;
};

typedef struct linkedlist_t *LinkedList;

typedef struct	{						/* for storing application meta-info */
	char	*name,
			*version,
			*description,
			*args;
	void	(*func)(void);
	int	fcgi_keepalive;
}	appinfo_t;

/***********************************************************************
	PUBLIC PROTOTYPES
 ***********************************************************************/

extern void	htdb_httpHeader(char *fmt, ...);
extern void	htdb_httpHeaderEnd(void);
extern void	htdb_httpExpires(time_t the_past);
extern void	htdb_doHeader(char *content_type);
extern void	htdb_doFooter(char *content_type);
extern void	htdb_httpRedirect(char *fmt, ...);

/*
	uncgi.c
 */
extern void	uncgi_stuffenv(char *);
extern void	uncgi_scanquery(char *);
extern void	uncgi_process_name_value(char *, char *);
extern void	uncgi(void);

/*
	ternary.c
 */
extern Pdata	ternary_newval(char *key, void *value, int sz);
extern Pdata	ternary_search(Pnode p, char *s);
extern void	ternary_insert(Pnode *p, char *s, Pdata val);
extern int	ternary_size(Pnode p, char *key);
extern Pdata	ternary_node(Pnode p, char *key);
extern void ternary_traverse(Pnode p);
extern void ternary_dupe(Pnode p, Pnode pp);
extern void	*ternary_value(Pnode p, char *s);
extern void ternary_destroy(Pnode p);

#define	ternary_store(r, key, value, sz) \
	ternary_insert(&(r), (key), ternary_newval((key), (void *)value, sz))

/*
	hash.c
 */
extern Hashtable	ht_create(void (*destructor)(void *), int size);
extern void			ht_destroy(Hashtable);
extern Bucket		ht_lookup(Hashtable, char *);
extern int			ht_size(Hashtable, char *);
extern void			*ht_value(Hashtable, char *);
extern Bucket		ht_traverse(Hashtable);
extern void			ht_init(Hashtable);
extern void			bucket_init(Bucket);
extern void			bucket_destroy(Bucket);
extern int			ht_define(Hashtable, char *, void *, int);
extern void			ht_duplicate(Hashtable, Hashtable);

/*
	web content types
 */
#define	CONTENT_TYPE_TEXT_HTML			"Content-type: text/html"
#define	CONTENT_TYPE_TEXT_XML			"Content-type: text/xml"
#define	CONTENT_TYPE_TEXT_PLAIN			"Content-type: text/plain"
#define	CONTENT_TYPE_IMAGE_GIF			"Content-type: image/gif"
#define	CONTENT_TYPE_IMAGE_JPEG			"Content-type: image/jpeg"
#define	CONTENT_TYPE_X_PN_REALAUDIO		"Content-type: audio/x-pn-realaudio"
#define	CONTENT_TYPE_AUDIO_MPEG			"Content-type: audio/mpeg"
#define	CONTENT_TYPE_APPLICATION_SMIL	"Content-type: application/smil"
#define	CONTENT_TYPE_REALTEXT			"Content-type: text/vnd.rn-realtext"
#define	CONTENT_TYPE_NON_CGI			"me? i'm just a program, me"
#define	CONTENT_TYPE_STATUS_204			"Status: 204\n"
#define	CONTENT_TYPE_EOH				"\n"

#define	FUNC_CALCULATES_SIZE		-1

/*
	handy equivalants
 */
#define	htdb_include			htdb_read
#define	htdb_print				htdb_parse

#ifdef	USE_TERNARY
	/*
		using ternary.c and ternary trees for general-purpose storage
	 */
	typedef	Pnode	spaceName;
	typedef	Pdata	spacePData;
	typedef	Tdata	spaceData;

	/*
		NOT YET COMPLETE
	 */
	#define	space_init(h, s, d)
	#define	space_initwalk(h)
	#define	space_walk(h)			ternary_traverse(h)
	#define	space_destroy(h)		ternary_destroy(h), (h) = NULL
	#define	space_lookup(h, k)		ternary_search((h), (k))
	#define	space_value(h, k)		ternery_value((h), (k))
	#define	space_size(h, a)		ternary_size((h), (a))
	#define	space_store(h, k, v, s)	ternary_store((h), (k), (v), (s))
	#define	space_duplicate(a, b)	ternary_dupe((a), (b))
	#define	data_init(d)
	#define	data_destroy(d)

#else

	/*
		using hash.c and hash tables for general-purpose storage
	 */
	typedef	Hashtable			spaceName;
	typedef	Bucket				spacePData;
	typedef	struct bucket_t		spaceData;

	/*
		hashtable init, walk and teardown macros
	 */
	#define	space_init(h, s, d)		if (!(h)) (h) = ht_create((d), (s))
	#define	space_initwalk(h)		ht_init(h)
	#define	space_walk(h)			ht_traverse(h)
	#define	space_destroy(h)		ht_destroy(h), (h) = NULL
	#define	space_lookup(h, k)		ht_lookup((h), (k))
	#define	space_value(h, k)		ht_value((h), (k))
	#define	space_size(h, a)		ht_size((h), (a))
	#define	space_store(h, k, v, s)	ht_define((h), (k), (void *)(v), (s))
	#define	space_duplicate(a, b)	ht_duplicate((a), (b))
	#define	data_init(d)			bucket_init((Bucket)d)
	#define	data_destroy(d)			bucket_destroy((Bucket)d)

#endif

/***********************************************************************
	BEGIN EXPERIMENTAL
 */
typedef	struct {
	int	len_gokey,
		fcgi_keepalive,
		buf_limit_tiny,
		buf_limit_small,
		buf_limit_medium,
		buf_limit_big,
		buf_limit_max,
		hashtabsize_medium,
		hashtabsize_big,
		hashtabsize_max;
}	htdb_constants_t;

typedef	struct {
	htdb_constants_t	constants;
	spaceName			staticSpace,
						resSpace,		/* per request storage area */
						tagSpace,		/* tag definitions */
						funcSpace,		/* script functions */
						cfuncSpace,		/* DSO functions */
						dbSpace,		/* database handles */
						fileSpace;		/* file pointers */
}	namespace_t;

/*
	each namespace should be allowed to have their
	own constants (hash sizes, gokey lens, fcgi loops, etc).
	so... the `namespace' storage object will store
	pointers to `namespace_t' objects.
 */
extern spaceName	nameSpace;

#define	name_initspace()		space_init(nameSpace, HASHTABSIZE_BIG, htdb_destroyNameSpace)
#define	name_initwalk()			space_initwalk(nameSpace)
#define	name_walkspace()		space_walk(nameSpace)
#define	name_clearspace()		space_destroy(nameSpace)
#define	name_size(a)			space_size(nameSpace, (a))
#define	name_store(k, v)		space_store(nameSpace, (k), (v), 1)
#define	name_value(k)			(char *)space_value(nameSpace, (k))
#define	name_lookup(k)			space_lookup(nameSpace, (k))

/*
	END EXPERIMENTAL
 ***********************************************************************/
/*
	GLOBAL variables
 */
extern	spaceName	staticSpace,	/* resources read from static.htdb */
					resSpace,		/* per request storage area */
					tagSpace,		/* tag definitions */
					funcSpace,		/* script functions */
					cfuncSpace,		/* DSO functions */
					dbSpace,		/* database handles */
					fileSpace;		/* file pointers */

/*
	macros for manipulating the static resource namespace
 */
#define	static_initspace()		space_init(staticSpace, HASHTABSIZE_BIG, htdb_destroyStringTuple)
#define	static_initwalk()		space_initwalk(staticSpace)
#define	static_walkspace()		space_walk(staticSpace)
#define	static_clearspace()		space_destroy(staticSpace)
#define	static_size(a)			space_size(staticSpace, (a))
#define	static_store(k, v)		space_store(staticSpace, (k), (v), 1)
#define	static_value(k)			(char *)space_value(staticSpace, (k))
#define	static_lookup(k)		space_lookup(staticSpace, (k))

/*
	macros for manipulating the core resource namespace
 */
#define	htdb_initspace()		space_init(resSpace, HASHTABSIZE_BIG, htdb_destroyStringTuple)
#define	htdb_initwalk()			space_initwalk(resSpace)
#define	htdb_walkspace()		space_walk(resSpace)
#define	htdb_clearspace()		space_destroy(resSpace)
#define	htdb_size(a)			space_size(resSpace, (a))
#define	htdb_store(k, v)		space_store(resSpace, (k), (v), 1)
#define	htdb_value(k)			(char *)space_value(resSpace, (k))
#define	htdb_binary(k, v, s)	space_store(resSpace, (k), (v), (0 - (s)))
#define	htdb_lookup(k)			space_lookup(resSpace, (k))

/*
	macros for accessing the (c-function) DSO storage area
 */
#define	cfunc_initspace()		space_init(cfuncSpace, HASHTABSIZE_SMALL, htdb_destroyFunctionPointer)
#define	cfunc_initwalk()		space_initwalk(cfuncSpace)
#define	cfunc_walkspace()		space_walk(cfuncSpace)
#define	cfunc_clearspace()		space_destroy(cfuncSpace)
#define	cfunc_store(k, v)		space_store(cfuncSpace, (k), (v), 0)
#define	cfunc_lookup(k)			space_lookup(cfuncSpace, (k))

/*
	macros for manipulating the database handles
 */
#define	db_initspace()			space_init(dbSpace, HASHTABSIZE_SMALL, htdb_destroyDatabase)
#define	db_initwalk()			space_initwalk(dbSpace)
#define	db_walkspace()			space_walk(dbSpace)
#define	db_clearspace()			space_destroy(dbSpace)
#define	db_store(k, v)			space_store(dbSpace, (k), (v), 0)
#define	db_lookup(k)			space_lookup(dbSpace, (k))

/*
	macros for manipulating the script function namespace
 */
#define	sfunc_initspace()		space_init(funcSpace, HASHTABSIZE_SMALL, htdb_destroyScriptFunction)
#define	sfunc_initwalk()		space_initwalk(funcSpace)
#define	sfunc_walkspace()		space_walk(funcSpace)
#define	sfunc_clearspace()		space_destroy(funcSpace)
#define	sfunc_store(k, v)		space_store(funcSpace, (k), (v), 0)
#define	sfunc_lookup(k)			space_lookup(funcSpace, (k))

/*
	macros for manipulating the user-defined tag namespace
 */
#define	tag_initspace()			space_init(tagSpace, HASHTABSIZE_SMALL, htdb_destroyStringTuple)
#define	tag_initwalk()			space_initwalk(tagSpace)
#define	tag_walkspace()			space_walk(tagSpace)
#define	tag_clearspace()		space_destroy(tagSpace)
#define	tag_store(k, v)			space_store(tagSpace, (k), (v), 0)
#define	tag_lookup(k)			space_lookup(tagSpace, (k))

/*
	macros for manipulating the user-defined file namespace
 */
#define	file_initspace()		space_init(fileSpace, HASHTABSIZE_SMALL, htdb_destroyFilePointer)
#define	file_initwalk()			space_initwalk(fileSpace)
#define	file_walkspace()		space_walk(fileSpace)
#define	file_clearspace()		space_destroy(fileSpace)
#define	file_store(k, v)		space_store(fileSpace, (k), (v), 0)
#define	file_lookup(k)			space_lookup(fileSpace, (k))

/*
	the public interfaces
 */
extern void	htdb_usage(appinfo_t *appinfo);
extern void	htdb_log(char *resource, char *aux_value);
extern bool_t	htdb_read(char *);
extern bool_t	htdb_main(int argc, char **argv, appinfo_t *);
extern void	htdb_page_handler(void);
extern void	htdb_setfunc(char *key, char *arglist, char *bd);

/*
	storage area tear-down stub functions
 */
extern void	htdb_destroyNameSpace(void *value);
extern void	htdb_destroyStringTuple(void *value);
extern void	htdb_destroyScriptFunction(void *value);
extern void	htdb_destroyFunctionPointer(void *value);
extern void	htdb_destroyDatabase(void *value);
extern void	htdb_destroyFilePointer(void *value);

/*
	accessors.c
 */
extern char	*static_setval(char *key, char *val);
extern int_t	static_setint(char *key, int_t val);
extern long_t	static_setlong(char *key, long_t val);
extern char	*static_getval(char *name);
extern int_t	htdb_setint(char *key, int_t val);
extern long_t	htdb_setlong(char *key, long_t val);
extern char	*htdb_setval(char *key, char *val);
extern void	*htdb_setbin(char *key, void *val, off_t size);
extern char	*htdb_setvarg(char *name, char *fmt, ...);
extern float	htdb_setfloat(char *key, float val, int precision);
extern char	*htdb_getval(char *name);
extern char	*htdb_getvargval(char *fmt, ...);
extern int_t	htdb_getvargint(char *fmt, ...);
extern long_t	htdb_getvarglong(char *fmt, ...);
extern void	*htdb_getbin(char *name);
extern char	*htdb_getarrayname(char *name, int index);
extern char	*htdb_setarrayval(char *name, int index, char *fmt, ...);
extern char	*htdb_getarrayval(char *name, int index);
extern int_t	htdb_setarrayint(char *name, int index, int_t value);
extern long_t	htdb_setarraylong(char *name, int index, long_t value);
extern int_t	htdb_getarrayint(char *name, int index);
extern long_t	htdb_getarraylong(char *name, int index);
extern char	*htdb_getobjdyname(char *name, int index, char *field);
extern char	*htdb_getobjname(char *name, int index, char *field);
extern char	*htdb_getobjval(char *name, int index, char *field);
extern int_t	htdb_getobjint(char *name, int index, char *field);
extern long_t	htdb_getobjlong(char *name, int index, char *field);
extern char	*htdb_setobjvarg(char *name, int index, char *field, char *fmt, ...);
extern char	*htdb_setobjval(char *name, int index, char *field, char *setto);
extern int_t	htdb_setobjint(char *name, int index, char *field, int_t setto);
extern long_t	htdb_setobjlong(char *name, int index, char *field, long_t setto);
extern float	htdb_setobjfloat(char *name, int index, char *field, float val, int precision);
extern float	htdb_setptrfloat(char *name, char *field, float val, int precision);
extern char	*htdb_cond(char *to, char *from);
extern void	htdb_loop(char *to, char *from);
extern void	htdb_append(char *to, char *from);
extern void	htdb_appendvarg(char *to, char *fmt, ...);
extern char	*htdb_getptrdyname(char *name, char *field);
extern char	*htdb_getptrval(char *name, char *field);
extern int_t	htdb_getptrint(char *name, char *field);
extern long_t	htdb_getptrlong(char *name, char *field);
extern int_t	htdb_setptrint(char *name, char *field, int_t setto);
extern long_t	htdb_setptrlong(char *name, char *field, long_t setto);
extern char	*htdb_setptrval(char *name, char *field, char *setto);
extern char	*htdb_setptrvarg(char *name, char *field, char *fmt, ...);

#define	htdb_getfloat(a)	atof(htdb_getval(a))
#define	htdb_getint(a)		atol(htdb_getval(a))
#define	htdb_getlong(a)		atoll(htdb_getval(a))
#define	htdb_getnum(a)		htdb_getint(a)
#define	htdb_checkval(a)	(*htdb_getval(a) != '\0')
#define	htdb_defined(a)		htdb_checkval(a)
#define	static_checkval(a)	(*static_getval(a) != '\0')
#define	htdb_equal(a, b)	(((b) && *(b) && strcasecmp(htdb_getval(a), (b)) == 0) ? 1 : 0)
#define	htdb_objequal(a, b, c, d)	(((d) && *(d) && strcasecmp(htdb_getobjval(a,b,c), (d)) == 0) ? 1 : 0)
#define	htdb_script(a)		htdb_expand(htdb_getval(a))

/*
	live.c
 */
extern int	live_eval_expr(char *str);
extern char	*live_getval(char *token);
extern void	thing_append(spaceData *b, char *expanded, int length, int growby);
extern void	thing_appendChar(spaceData *b, char expanded, int growby);
extern char	*htdb_expand(char *str);
extern char	*htdb_barewordexpand(char *str);
extern void	htdb_parse(char *str);

/*
	session.c
 */
extern void	htdb_SetFromPathInfo(char *lookup, char *val);
extern int_t	htdb_fetchHitCount(int_t domain_id, char *resource, char *table);
extern bool_t	htdb_handleSessioning(void);

/*
	util.c
 */
typedef	struct	{						/* for storing script functions */
	char	*body,
			**args,
			**defaults;
	int		num_args;
}	sfunc_t;

typedef	struct	{						/* for storing file pointers */
	char	*path,
			*mode;
	FILE	*fp;
}	filep_t;

extern char	*htdb_ordinalSuffix(int in);
extern char	*prepareStringForDatabase(char *);
extern char	*prepareStringForBBSDatabase(char *);
extern char	*prepareStringFromDatabase(char *);
extern char	*prepareBinaryFromDatabase(char *in, off_t *out_bytes);
extern char	*prepareBinaryForDatabase(char *in, off_t in_bytes, off_t *out_bytes);
extern char	*filterProgramArguments(char *in);
extern char	*pretty(char *ugly);
extern char	*prettyState(char *state);
extern char	*space2underscore(char *in);
extern char	*newline2br(char *in);
extern char	*br2newline(char *in);
extern bool_t	sendmail(char *, char *);
extern bool_t	amRoot(void);
extern char	*encode(char *source);
extern char	*makeFileSystemSafe(char *in);
extern char	**constructArgumentList(char *in, char separator, int *num);
extern void	destroyArgumentList(char ***args);
extern void	deleteSessionData(int_t session_id, char *name);
extern void	storeSessionData(int_t session_id, char *name, char *value);
extern int	fetchSessionData(int_t session_id, char *name);
extern void	deleteUserData(int_t user_id, char *name);
extern void	storeUserData(int_t user_id, char *name, char *value);
extern void	fetchUserData(int_t user_id, char *name);
extern void	fetchUserAccountTypes(char *prefix, int_t user_id);
extern void	fetchNamedUserData(char *prefix, int_t user_id, char *name);
extern bool_t	isTag(char *tag, char *in, char **gottag);
extern char	*reverseString(char *in);
extern int	myIndexOf(char *table, char *field, char *value);
extern char *htdb_unfilter(char *in);
extern char *htdb_unfilter_unsafe(char *in);
extern char	*htdb_unfilter_nodoublequotes(char *in);
extern char	*genGoKey(void);
extern char	*genAlnumKey(char *table, char *field, int keylen);
extern void	htdb_objtransfer(char *prefix_dest, int index_dest, char *prefix_src, int index_src);
extern char	*htdb_mapStr(char *pattern1, char *pattern2, char *input);
extern int_t	htdb_fetchVerbage(char *prefix, char *lookup);
extern char	*htdb_lookupVerbage(char *lookup);
extern void	htdb_emailVerbage(char *lookup);
extern char	*sstrcat(char *dest, const char *src, size_t n);
extern bool_t	htdb_inList(char *src, char *in, bool_t exact);
extern bool_t	htdb_inLine(char *src, char *in, bool_t exact);
extern int_t	htdb_random(int_t max);
extern char	*html_safe_trunc(char *string, int len);
extern int	htdb_splitArgs(char *src, char *sep, char ***out, int max_args);
extern void	htdb_deleteArgs(char **args, int num);
extern bool_t	isValidEmailAddress(char *email);
extern bool_t	isStringNumeric(char *in);
extern char	*htdb_stripWhitespace(char *chunk);
extern char	*htdb_strcasestr(const char *str, const char *search);
extern char	*strcasestr(const char *s1, const char *s2);
extern char	*htdb_cleanLines(char *in);
extern char	genAlnum(void);
extern char	*literalizeCommas(char *in);
extern char	*literalizeQuotes(char *in);
extern int	removeDirectory(char *dir);
extern int	lookupIndexByName(char *prefix, char *name);
extern bool_t	parseFilterFunction(char *source, char **filterThis, int *index);
extern char *ttgDetail(char *res, int_t domain_id, int precision, int bars, int limit);
extern void	htdb_objsort(char *prefix, char *key);
extern char	*htdb_prettyNumber(char *in);
extern char	*ccNumber(char *in);
extern int	ccValid(char *in);
extern void	htdb_srand(void);
extern char	*htdb_system(char *cmd);
extern char	*htdb_uname(char *in);
extern bool_t	isUnconfirmed(void);
extern bool_t	isConfirmed(void);
extern bool_t	isUser(void);
extern bool_t	isAdmin(void);
extern int	uncgi_htoi(unsigned char *s);
extern void	uncgi_url_unescape(unsigned char *str);
extern char	*url_escape(char *in);
extern char	*htdb_ttg(char *in);
extern int	rtoi(char *rom);
extern void	itor(int num, char **str);
extern char	*braille(char *in);
/*
	a null-terminating strncpy()
	ONLY WORKS FOR STATIC DESTINATION STORAGE
 */
#define	sstrncpy(out, in) \
		strncpy((out), (in), sizeof(out)), (out)[sizeof(out) - 1] = '\0'
/*
	if no strncpy()/snprintf()

extern char	*strncpy(char *out, const char *in, size_t bytes);
extern int	snprintf(char *out, size_t bytes, const char *fmt, ...);
*/

/*
	dt.c
 */
extern bool_t	isLeapYear(int year);
extern char	*prettySeconds(long tm);
extern char	*prettySecondsTerse(long tm);
extern time_t	fetchTimestamp(void);
extern int	fetchDaysToEndOfMonth(long timestamp);
extern char	*htdb_date(char *in);
extern char	*htdb_parseUnix(char *in);
extern char	*htdb_parseTime(char *in);
extern char	*htdb_parseDate(char *in);
extern char	*htdb_parseDateTime(char *in);
extern char	*htdb_parseMonthNum(char *in);
extern char	*htdb_parseMonth(char *in);
extern char	*htdb_parseMon(char *in);
extern char	*htdb_parseDay(char *in);
extern char	*htdb_parseYear(char *in);
extern char	*htdb_parseHour(char *in);
extern char	*htdb_parseMilHour(char *in);
extern char	*htdb_parseSecond(char *in);
extern char	*htdb_parseMinute(char *in);
extern char	*htdb_parseAMPM(char *in);
extern char	*htdb_parseWeekday(char *in);

extern char	*htdb_month2num(char *in);
extern char	*htdb_num2mon(char *in);
extern char	*htdb_num2month(char *in);
extern char	*htdb_fetchNextMonth(char *in);
extern char	*htdb_fetchPrevMonth(char *in);

/*
	log.c
 */
extern void	log_page(char *session_id, char *domain_id, char *user_id, char *affiliate_id, char *browser_id, char *resource, char *value);
extern void	log_ttg(char *thing);

/*
	dso.c
 */
#define	GENERIC_SUCCESS_STRING	"<br><b>SUCCESS</b>"
#define	GENERIC_FAILURE_STRING	"<br><b>FAILURE</b>"

extern bool_t	dso_loadFiles(char *dsoList);
extern void	dso_unloadFiles(void);
extern void	dso_fetchFunction(char *, char *(**)(char *));
extern char	*DSO_RETURNS_SUCCESS_CHALLENGED(char *msg);
#define	DSO_RETURNS_SUCCESS_ASIS(msg)		(msg)
#define	DSO_RETURNS_SUCCESS(msg)			strdup(msg)

/*
	crypto.c
 */
extern char	*do_encryptor(char *src, char *key_str, char *algo);
extern char	*do_decryptor(char *src, char *key_str, char *algo);
extern char	*htdb_encryptor(char *prefix, char *source);
extern char	*htdb_decryptor(char *source);

/*
	base64.c
 */
extern char	*base64_encode(const char *src, BASE64Uint32 srclen, char *dest);
extern char	*base64_decode(const char *src, BASE64Uint32 srclen);

/*
	upload.c
 */
extern void doUpLoad(void);
extern void destroyUpLoadInfo(void);
extern int getUpLoadInfo(char *dir, LinkedList *nv, LinkedList *files, char *errstring);
extern void setUpLoadInfo(char *dir, LinkedList nv, LinkedList files);

/*
	db.c
 */
#if	(defined(USE_DBI) && defined(HAVE_LIBDBI))

	#include	<dbi.h>

	typedef	dbi_conn	_dbHandle;

	#define	_dbQuery			dbi_conn_query
	#define	_dbBinaryQuery		dbi_conn_query_null

#endif

#if	(defined(USE_MYSQL) && defined(HAVE_LIBMYSQLCLIENT))

	#include	<mysql.h>

	typedef	MYSQL_RES	_dbResultHandle;
	typedef	MYSQL_ROW	_dbResultRow;
	typedef	MYSQL_FIELD	_dbField;
	typedef	MYSQL	*	_dbHandle;
	#define	_dbFetchRow		mysql_fetch_row
	#define	_dbFreeResultHandle	mysql_free_result
	#define	_dbQuery			mysql_query
	#define	_dbBinaryQuery		mysql_real_query
	#define	_dbFetchErrorString	mysql_error

#endif

#if	(defined(USE_DATABASE))
	extern _dbResultHandle	*dbSQLQuery(char *cmd, char **handle, int *num_row, int *success);
#else
	#define _dbHandle   void    *
	#define	_dbResultHandle	void
#endif

extern bool_t	doConnect(void);
extern int	dbQuery(char *fmt, ...);
extern int_t	dbInsert(char *fmt, ...);
extern int	dbUpdate(char *fmt, ...);
extern int	dbDelete(char *fmt, ...);
extern int	dbProcessByRow(char *cmd, char *resource);
extern int	dbCallbackByRow(void (*fptr) (int, int, char **, char **), char *fmt, ...);
extern int	dbGetRowByKey(char *table, char *fmt, ...);
extern int	dbQueryBinary(char *cmd, unsigned int length);
extern time_t	fetchDatabaseTimestamp(void);
extern _dbHandle	dbGetSocket(char *handle);
extern void	dbSync();
extern void	dbUnsync();

/*
	debug.c
 */
#define	DEBUG_LEVEL_FATAL	(0x1 << 1)
#define	DEBUG_LEVEL_ERROR	(0x1 << 2)
#define	DEBUG_LEVEL_WARNING	(0x1 << 3)
#define	DEBUG_LEVEL_INFO	(0x1 << 4)
#define	DEBUG_LEVEL_DEBUG	(0x1 << 5)

extern void	debug_it(int mask, char *label, char *msg, char **out_buf);
extern void	log_debug(char *label, char *fmt, ...);
extern void	log_info(char *label, char *fmt, ...);
extern void	log_warning(char *label, char *fmt, ...);
extern void	log_error(char *label, char *fmt, ...);
extern void	log_fatal(char *label, char *fmt, ...);
extern void	log_intro(void);
extern void	log_outro(void);
extern void	log_file(char *file, char *fmt, ...);
extern void	debug_resDump(char *file);
extern void	profile_begin(void);
extern double 	profile_end(void);
extern void	log_profile_end(char *mesg);

extern void	dump(char *str);

/*
	forms.c
 */
extern void	setFormErrors(char *resource, int *index);
extern int	setErrors(int set, char *name, int row, char *resource, int *index);
extern int	setEmail(char *prefix, char *name, int required, int row, char *resource, int *index);
extern int	setForm(char *prefix, char *name, int required, int row, char *resource, int *index);
extern int	setCheckbox(char *prefix, char *name, int required, int row, char *resource, int *index);
extern int	setRadio(char *prefix, char *name, int required, int row, char *resource, int *index);
extern int	setIntOption(char *prefix, char *name, int required, int row, char *resource, int *index);
extern int	setLimitedIntOption(char *prefix, char *name, int required, int row, char *resource, int *index);
extern int	setTextOption(char *prefix, char *name, int required, int row, char *resource, int *index);
extern int	setScreenName(char *prefix, char *name, int required, int row, char *resource, int *index);
extern int	setYear(char *prefix, char *name, int required, int row, char *resource, int *index);
extern bool_t	containsCensoredWords(char *val);

/*
	graphics.c
 */
extern bool_t	imageStore(char *table, int_t user_id, char *name, char *buf, char *ext, off_t media_bytes, int w, int h);
extern bool_t	imageScaleAndStore(char *table, int_t user_id, char *name, char *dir, char *file, int width, int height);
extern bool_t	imageGetDimensions(char *path, int *width, int *height);
extern char		*imageGetContentByExtension(char *ext);
extern char		*imageGetContentByData(char *buf);
extern char		*imageGetExtensionByData(char *buf);

/*
	google.c
 */
extern char	*doGoogleSearch(char *key, char *q, int start, int maxResults, bool_t filter, char *restric, bool_t safeSearch, char *lr);

/*
	md5.c
 */
extern void md5_digest(char *str, unsigned char digest[16], long offset, long bytes);

#endif
